using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

// HZMCUACOLLISIONS v1.0
//
// Improve CUA collisions when you need to interact with it

namespace VAMANeighborsPlugin
{
    public class VAMANeighbors : MVRScript
    {
        public static string PLUGIN_PATH;

		public Transform CUARoot;
		
		// Game Objects and components	
		private bool cuaInitialized = false;
		
		// VAM UI
		private JSONStorableString stateInfos;
		
		public JSONStorableFloat MinPauseTask;
		public JSONStorableFloat MaxPauseTask;
		public JSONStorableFloat NeighborArousalProbability;
		
		public JSONStorableFloat NeighborVolume;
		
		public JSONStorableBool ShowDebug;
		
		public JSONStorableFloat Neighbor1OffsetX;
		public JSONStorableFloat Neighbor1OffsetY;
		public JSONStorableFloat Neighbor1OffsetZ;
		
		public JSONStorableFloat Neighbor2OffsetX;
		public JSONStorableFloat Neighbor2OffsetY;
		public JSONStorableFloat Neighbor2OffsetZ;
		
		public JSONStorableFloat Neighbor3OffsetX;
		public JSONStorableFloat Neighbor3OffsetY;
		public JSONStorableFloat Neighbor3OffsetZ;
			
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;
		
		// Others
		private UIDynamicTextField stateinfosTextfield;
		
		private bool _atmosphereEnabled = true;
	
		private GameObject GoNeighbor1;
		private GameObject GoNeighbor2;
		private GameObject GoNeighbor3;

		private List<string> neighbor1TasksPaths;
		private List<string> neighbor2TasksPaths;
		private List<string> neighbor3TasksPaths;
		
		private List<string> moviesListPaths;
		private List<string> tvShowsListPaths;

		private List<string> neighbor1SexPaths;
		private List<string> neighbor2SexPaths;
		private List<string> neighbor3SexPaths;		
		
		private List<AudioClip> neighbor1Tasks;
		private List<AudioClip> neighbor2Tasks;
		private List<AudioClip> neighbor3Tasks;

		private List<AudioClip> moviesList;
		private List<AudioClip> tvShowsList;

		private List<AudioClip> neighbor1Sex;
		private List<AudioClip> neighbor2Sex;
		private List<AudioClip> neighbor3Sex;

		private bool atmosphereRoutineEnabled = false;
		private AudioSource AsNeighbor1;
		private AudioSource AsNeighbor2;
		private AudioSource AsNeighbor3;

		private List<AudioClip> neightbor1DailyTasks;
		private List<AudioClip> neightbor2DailyTasks;
		private List<AudioClip> neightbor3DailyTasks;
		
		// Assetbundle
		private Request audioBundleRequest = null;	
			
		public override void Init()
        {			 
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add VAMANeighbors on a Custom Unity Asset Atom");
                    return;
                }
				
				PLUGIN_PATH = GetPluginPath(this);	
							
				MinPauseTask = new JSONStorableFloat("Pause between tasks (min)", 5f, MinPauseTaskCallback, 1f, 60f);
				CreateSlider(MinPauseTask, false);
				
				MaxPauseTask = new JSONStorableFloat("Pause between tasks (max)", 15f, MaxPauseTaskCallback, 1f, 60f);
				CreateSlider(MaxPauseTask, false);
				
				NeighborArousalProbability = new JSONStorableFloat("Neighbor arousal probability", 0.3f, NeighborArousalProbabilityCallback, 0f, 1f);
				CreateSlider(NeighborArousalProbability, false);

				NeighborVolume = new JSONStorableFloat("Volume", 1f, NeighborVolumeCallback, 0f, 1f);	
				CreateSlider(NeighborVolume, false);
				
				ShowDebug = new JSONStorableBool("Show debug", false, ShowDebugCallback);
				CreateToggle(ShowDebug, false);
				
				CreateSpacer();
				
				createStaticDescriptionText("Global options","<color=#000><size=25><b>NEIGHBORS POSITIONS</b></size></color>",false,40);

				Neighbor1OffsetX = new JSONStorableFloat("Neighbor 1 offset X", -6f, Neighbor1OffsetXCallback, -25f, 25f);
				CreateSlider(Neighbor1OffsetX, false);
				
				Neighbor1OffsetY = new JSONStorableFloat("Neighbor 1 offset Y", 0f, Neighbor1OffsetYCallback, -25f, 25f);
				CreateSlider(Neighbor1OffsetY, false);
				
				Neighbor1OffsetZ = new JSONStorableFloat("Neighbor 1 offset Z", 0f, Neighbor1OffsetZCallback, -25f, 25f);
				CreateSlider(Neighbor1OffsetZ, false);
				
				Neighbor2OffsetX = new JSONStorableFloat("Neighbor 2 offset X", 6f, Neighbor2OffsetXCallback, -25f, 25f);
				CreateSlider(Neighbor2OffsetX, false);
				
				Neighbor2OffsetY = new JSONStorableFloat("Neighbor 2 offset Y", 0f, Neighbor2OffsetYCallback, -25f, 25f);
				CreateSlider(Neighbor2OffsetY, false);
				
				Neighbor2OffsetZ = new JSONStorableFloat("Neighbor 2 offset Z", 0f, Neighbor2OffsetZCallback, -25f, 25f);
				CreateSlider(Neighbor2OffsetZ, false);
				
				Neighbor3OffsetX = new JSONStorableFloat("Neighbor 3 offset X", 0f, Neighbor3OffsetXCallback, -25f, 25f);
				CreateSlider(Neighbor3OffsetX, false);
				
				Neighbor3OffsetY = new JSONStorableFloat("Neighbor 3 offset Y", 6f, Neighbor3OffsetYCallback, -25f, 25f);
				CreateSlider(Neighbor3OffsetY, false);
				
				Neighbor3OffsetZ = new JSONStorableFloat("Neighbor 3 offset Z", 0f, Neighbor3OffsetZCallback, -25f, 25f);
				CreateSlider(Neighbor3OffsetZ, false);
				
				stateInfos = new JSONStorableString("Stateinfos", "");
				stateinfosTextfield = CreateTextField(stateInfos, true);
				stateinfosTextfield.height = 250.0f;
				
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help - Neighbors</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>IMPORTANT:</b> If you switch asset in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"<b>Pause between tasks:</b> the time between each sound played in the neighborhood.\n\n" +
					"<b>Neighbor arousal probability:</b> When a neighbor has sex, it has a probability to arouse another neighbor. 1 means 100% chance.\n\n" +
					"<b>Show debug:</b> enable visual help to show the neighbors positions. Useful when you want to tweak the offsets. The tiny sphere should be outside of the play area (or the appartment if you prefer, behind a wall) and the big sphere around is the maximum distance at which the neighbor can be heard.\n\n" +
					"<b>Neighbors position (offsets):</b> allow to tweak the position of each neighbor.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 950.0f;

				InitPaths();
				
				RegisterFloat(MinPauseTask);
				RegisterFloat(MaxPauseTask);
				RegisterFloat(NeighborArousalProbability);
				
				RegisterFloat(Neighbor1OffsetX);
				RegisterFloat(Neighbor1OffsetY);
				RegisterFloat(Neighbor1OffsetZ);
				
				RegisterFloat(Neighbor2OffsetX);
				RegisterFloat(Neighbor2OffsetY);
				RegisterFloat(Neighbor2OffsetZ);
				
				RegisterFloat(Neighbor3OffsetX);
				RegisterFloat(Neighbor3OffsetY);
				RegisterFloat(Neighbor3OffsetZ);
			}
            catch(Exception e)
            {
                SuperController.LogError("VAMANeighbors - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update() {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && cuaInitialized == false ) {
				initLoops++;
				InitCUA();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("VAMANeighbors : CUA could not be initialized properly after 3600 tries. Please select a compatible CUA and reload the script.");
				initLoops++; // To avoid spamming the log
			}
		}
		

		
		// **************************
		// Functions
		// **************************
				
		private bool InitCUA() {
			if( cuaInitialized == true ) return true;
			
			CUARoot = getCUARoot( containingAtom.reParentObject );
			if( CUARoot == null ) {
				return false;
			}
			
			Transform trN1 = getObjectNamed(CUARoot,"neighbor1");
			GoNeighbor1 = trN1.gameObject;
			AsNeighbor1 = trN1.gameObject.GetComponent<AudioSource>();
			
			Transform trN2 = getObjectNamed(CUARoot,"neighbor2");
			GoNeighbor2 = trN2.gameObject;
			AsNeighbor2 = trN2.gameObject.GetComponent<AudioSource>();
			
			Transform trN3 = getObjectNamed(CUARoot,"neighbor3");
			GoNeighbor3 = trN3.gameObject;
			AsNeighbor3 = trN3.gameObject.GetComponent<AudioSource>();
			
			if( AsNeighbor1 && AsNeighbor2 && AsNeighbor3 ) {
				cuaInitialized = true;
				logDebug("VAMANeighbors : Init successful");
				
				// Loading the bundle
				Request request = new AssetLoader.AssetBundleFromFileRequest {path = PLUGIN_PATH + "/vamatmosphere_neighbors.audiobundle", callback = OnAudioBundleLoaded};
				AssetLoader.QueueLoadAssetBundleFromFile(request);
			}
		
			return false;

		}
		
		protected void UpdateDebugDisplay() {
			if ( true ) {
				//GoRZDebugRadiusMin.transform.localScale = new Vector3( ZoneSizeMin.val, ZoneSizeMin.val, ZoneSizeMin.val );
				//GoRZDebugRadiusMax.transform.localScale = new Vector3( ZoneSizeMax.val, ZoneSizeMax.val, ZoneSizeMax.val );
			}
		}
		
		protected void InitPaths() {
			// Creating assets paths list
			neighbor1TasksPaths = new List<string>();
			neighbor1TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_babycrying01.ogg");
			neighbor1TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_fight01.ogg");
			neighbor1TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_fight02.ogg");
			neighbor1TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_kids.ogg");
			neighbor1TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_movie01.ogg");
			neighbor1TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_vacuum.ogg");
			neighbor1TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_walk_long.ogg");
			neighbor1TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_walk_short.ogg");

			neighbor2TasksPaths = new List<string>();
			neighbor2TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_construction01.ogg");
			neighbor2TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_construction02.ogg");
			neighbor2TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_dog01.ogg");
			neighbor2TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_dog02.ogg");
			neighbor2TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_tvshow01.ogg");
			neighbor2TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_walking01.ogg");
			neighbor2TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_walk_short.ogg");
			
			neighbor3TasksPaths = new List<string>();
			neighbor3TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n3_party01.ogg");
			neighbor3TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n3_party02.ogg");
			neighbor3TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n3_playingguitar01.ogg");
			neighbor3TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n3_talking01.ogg");
			neighbor3TasksPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_walking01.ogg");
			
			moviesListPaths = new List<string>();
			moviesListPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_movie01.ogg");
			moviesListPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_movie02.ogg");
			
			tvShowsListPaths = new List<string>();
			tvShowsListPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_tvshow01.ogg");
			tvShowsListPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_tvshow02.ogg");
			tvShowsListPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_tvshow03.ogg");
			
			neighbor1SexPaths = new List<string>();
			neighbor1SexPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_sex01.ogg");
			neighbor1SexPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n1_sex02.ogg");
			
			neighbor2SexPaths = new List<string>();
			neighbor2SexPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_sex01.ogg");
			neighbor2SexPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n2_sex02.ogg");
			
			neighbor3SexPaths = new List<string>();
			neighbor3SexPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n3_sex01.ogg");
			neighbor3SexPaths.Add("Assets/VAMAtmosphere/audio_neighbors/n3_sex02.ogg");
		}
		
		private AudioClip getMovieAudioclip() {
			int mIndex = UnityEngine.Random.Range(0, moviesList.Count);
			return moviesList[mIndex];
		}

		private AudioClip getTVShowAudioclip()
		{
			int mIndex = UnityEngine.Random.Range(0, tvShowsList.Count);
			return tvShowsList[mIndex];
		}
		
		private float playNeighborSex(int neighborIndex)
		{
			List<AudioClip> selectedSex;
			AudioSource asNeighbor;
			AudioClip clipToPlay;

			switch (neighborIndex)
			{
				case 2:
					selectedSex = neighbor2Sex;
					asNeighbor = AsNeighbor2;
					break;
				case 3:
					selectedSex = neighbor3Sex;
					asNeighbor = AsNeighbor3;
					break;
				default:
					selectedSex = neighbor1Sex;
					asNeighbor = AsNeighbor1;
					break;
			}

			int sexIndex = UnityEngine.Random.Range(0, selectedSex.Count);
			clipToPlay = selectedSex[sexIndex];
			asNeighbor.clip = clipToPlay;
			asNeighbor.Play();

			float waitTime = selectedSex[sexIndex].length + UnityEngine.Random.Range(1f, 3f);

			return waitTime;
		}

		// **************************
		// Callbacks & similar
		// **************************
		private void OnAudioBundleLoaded(Request aRequest) {
			try
            {
				audioBundleRequest = aRequest;
				
				neighbor1Tasks = new List<AudioClip>();
				foreach( string assetPath in neighbor1TasksPaths ) {
					AudioClip ac = audioBundleRequest.assetBundle.LoadAsset<AudioClip>(assetPath);
					neighbor1Tasks.Add( ac );
				}
				
				neighbor2Tasks = new List<AudioClip>();
				foreach( string assetPath in neighbor2TasksPaths ) {
					AudioClip ac = audioBundleRequest.assetBundle.LoadAsset<AudioClip>(assetPath);
					neighbor2Tasks.Add( ac );
				}

				neighbor3Tasks = new List<AudioClip>();
				foreach( string assetPath in neighbor3TasksPaths ) {
					AudioClip ac = audioBundleRequest.assetBundle.LoadAsset<AudioClip>(assetPath);
					neighbor3Tasks.Add( ac );
				}

				moviesList = new List<AudioClip>();
				foreach( string assetPath in moviesListPaths ) {
					AudioClip ac = audioBundleRequest.assetBundle.LoadAsset<AudioClip>(assetPath);
					moviesList.Add( ac );
				}
				
				tvShowsList = new List<AudioClip>();
				foreach( string assetPath in tvShowsListPaths ) {
					AudioClip ac = audioBundleRequest.assetBundle.LoadAsset<AudioClip>(assetPath);
					tvShowsList.Add( ac );
				}

				neighbor1Sex = new List<AudioClip>();
				foreach( string assetPath in neighbor1SexPaths ) {
					AudioClip ac = audioBundleRequest.assetBundle.LoadAsset<AudioClip>(assetPath);
					neighbor1Sex.Add( ac );
				}
				
				neighbor2Sex = new List<AudioClip>();
				foreach( string assetPath in neighbor2SexPaths ) {
					AudioClip ac = audioBundleRequest.assetBundle.LoadAsset<AudioClip>(assetPath);
					neighbor2Sex.Add( ac );
				}

				neighbor3Sex = new List<AudioClip>();
				foreach( string assetPath in neighbor3SexPaths ) {
					AudioClip ac = audioBundleRequest.assetBundle.LoadAsset<AudioClip>(assetPath);
					neighbor3Sex.Add( ac );
				}
				
				// The current daily tasks to do
				neightbor1DailyTasks = new List<AudioClip>(neighbor1Tasks);
				neightbor2DailyTasks = new List<AudioClip>(neighbor2Tasks);
				neightbor3DailyTasks = new List<AudioClip>(neighbor3Tasks);
				
				StartCoroutine("LoopAtmosphere");
				
				UpdateNeighborsSettings();
			}
            catch(Exception e)
            {
                SuperController.LogError("VAMAAmbience - Exception caught: " + e);
            }
		}
		
		protected void UpdateNeighborsSettings() {
			Neighbor1OffsetXCallback( Neighbor1OffsetX.val );
			Neighbor2OffsetXCallback( Neighbor2OffsetX.val );
			Neighbor3OffsetXCallback( Neighbor3OffsetX.val );
			
			ShowDebugCallback( ShowDebug.val );
		}
		
		protected void MinPauseTaskCallback(float value) {}
		protected void MaxPauseTaskCallback(float value) {}
		protected void NeighborArousalProbabilityCallback(float value) {}
		
		protected void NeighborVolumeCallback(float volume) {
			if( AsNeighbor1 && AsNeighbor2 && AsNeighbor3 ) {
				AsNeighbor1.volume = volume;
				AsNeighbor2.volume = volume;
				AsNeighbor3.volume = volume;
			}
		}
		
		protected void ShowDebugCallback(bool enabled) {
			if( CUARoot ) {				
				toggleDebugElems(CUARoot, enabled);
				UpdateDebugDisplay();
			}
		}
		
		// Offset Neighbor1
		protected void Neighbor1OffsetXCallback(float pos) {
			if( CUARoot && GoNeighbor1 ) {				
				GoNeighbor1.transform.position = new Vector3(pos,Neighbor1OffsetY.val,Neighbor1OffsetZ.val);
			}
		}
		
		protected void Neighbor1OffsetYCallback(float pos) {
			if( CUARoot && GoNeighbor1 ) {				
				GoNeighbor1.transform.position = new Vector3(Neighbor1OffsetX.val,pos,Neighbor1OffsetZ.val);
			}
		}
		
		protected void Neighbor1OffsetZCallback(float pos) {
			if( CUARoot && GoNeighbor1 ) {				
				GoNeighbor1.transform.position = new Vector3(Neighbor1OffsetX.val,Neighbor1OffsetY.val,pos);
			}
		}
		
		// Offset Neighbor2
		protected void Neighbor2OffsetXCallback(float pos) {
			if( CUARoot && GoNeighbor2 ) {				
				GoNeighbor2.transform.position = new Vector3(pos,Neighbor2OffsetY.val,Neighbor2OffsetZ.val);
			}
		}
		
		protected void Neighbor2OffsetYCallback(float pos) {
			if( CUARoot && GoNeighbor2 ) {				
				GoNeighbor2.transform.position = new Vector3(Neighbor2OffsetX.val,pos,Neighbor2OffsetZ.val);
			}
		}
		
		protected void Neighbor2OffsetZCallback(float pos) {
			if( CUARoot && GoNeighbor2 ) {				
				GoNeighbor2.transform.position = new Vector3(Neighbor2OffsetX.val,Neighbor2OffsetY.val,pos);
			}
		}
		
		// Offset Neighbor3
		protected void Neighbor3OffsetXCallback(float pos) {
			if( CUARoot && GoNeighbor3 ) {				
				GoNeighbor3.transform.position = new Vector3(pos,Neighbor3OffsetY.val,Neighbor3OffsetZ.val);
			}
		}
		
		protected void Neighbor3OffsetYCallback(float pos) {
			if( CUARoot && GoNeighbor3 ) {				
				GoNeighbor3.transform.position = new Vector3(Neighbor3OffsetX.val,pos,Neighbor3OffsetZ.val);
			}
		}
		
		protected void Neighbor3OffsetZCallback(float pos) {
			if( CUARoot && GoNeighbor3 ) {				
				GoNeighbor3.transform.position = new Vector3(Neighbor3OffsetX.val,Neighbor3OffsetY.val,pos);
			}
		}		
		
		// **************************
		// COROUTINES
		// **************************
		private IEnumerator LoopAtmosphere() 
		{
			var loop = 1;
			atmosphereRoutineEnabled = true;
			
			// Initial pause
			float firstPauseDuration = UnityEngine.Random.Range(MinPauseTask.val, MaxPauseTask.val);
			logToState("<b>INITIAL PAUSE:</b>" + Mathf.Round(firstPauseDuration) + "secs");
			yield return new WaitForSeconds(firstPauseDuration);
			
			while (_atmosphereEnabled == true)
			{
				List<AudioClip> selectedTasks;
				AudioSource asNeighbor;
				AudioClip clipToPlay;
				int neighborIndex;
				float waitTime;
				float pauseDuration;

				logToState("<b>DAY " + loop + "</b>");
				
				// 3 Tasks per day
				for (int numTask = 0; numTask < 3; numTask++)
				{
					// Getting the neighbor that should make noice
					neighborIndex = UnityEngine.Random.Range(1, 4);
					
					switch (neighborIndex)
					{
						case 2:
							selectedTasks = neightbor2DailyTasks;
							asNeighbor = AsNeighbor2;
							break;
						case 3:
							selectedTasks = neightbor3DailyTasks;
							asNeighbor = AsNeighbor3;
							break;
						default:
							selectedTasks = neightbor1DailyTasks;
							asNeighbor = AsNeighbor1;
							break;
					}

					int taskIndex = UnityEngine.Random.Range(0, selectedTasks.Count);
					clipToPlay = selectedTasks[taskIndex];

					// If the clip is a movie or a tvshow, let's get a random one from the pool
					if (clipToPlay.name == "n1_movie01")
					{
						clipToPlay = getMovieAudioclip();
					} else if (clipToPlay.name == "n2_tvshow01") {
						clipToPlay = getTVShowAudioclip();
					}

					asNeighbor.clip = clipToPlay;
					asNeighbor.Play();
					pauseDuration = UnityEngine.Random.Range(MinPauseTask.val, MaxPauseTask.val);
					waitTime = selectedTasks[taskIndex].length + pauseDuration;

					logToState("<b>Neighbor " + neighborIndex + ":</b> " + selectedTasks[taskIndex].name + "\n<b>Pause duration:</b> " + Mathf.Round(pauseDuration) + "secs");

					selectedTasks.RemoveAt(taskIndex);

					yield return new WaitForSeconds(waitTime);
				}

				// Setting the list of neighbors
				List<int> neighborWhoCanHaveSex = new List<int>();
				neighborWhoCanHaveSex.Add(1);
				neighborWhoCanHaveSex.Add(2);
				neighborWhoCanHaveSex.Add(3);

				// As long as the list is containing something, we loop
				while( neighborWhoCanHaveSex.Count > 0 )
				{
					int nIndex = UnityEngine.Random.Range(0, neighborWhoCanHaveSex.Count);
					// By default, with max count, we have sex at night (of course)
					if (neighborWhoCanHaveSex.Count == 3)
					{
						waitTime = playNeighborSex(neighborWhoCanHaveSex[nIndex]);
						logToState("<b>Neighbor " + neighborWhoCanHaveSex[nIndex] + "</b> is having sex tonight.");
						yield return new WaitForSeconds(waitTime);
					}
					// Or else, we check if the neighbor have been aroused
					else {
						float arousedRnd = UnityEngine.Random.Range(0f, 1f);
						if(arousedRnd < NeighborArousalProbability.val)
						{
							waitTime = playNeighborSex(neighborWhoCanHaveSex[nIndex]);
							logToState("<b>Neighbor " + neighborWhoCanHaveSex[nIndex] + " aroused!</b> Let's have sex.");
							yield return new WaitForSeconds(waitTime);
						}
					}

					neighborWhoCanHaveSex.RemoveAt(nIndex);
				}

				logToState("New day starting, resetting tasks.");
				neightbor1DailyTasks = new List<AudioClip>(neighbor1Tasks);
				neightbor2DailyTasks = new List<AudioClip>(neighbor2Tasks);
				neightbor3DailyTasks = new List<AudioClip>(neighbor3Tasks);
				loop++;


				yield return new WaitForSeconds(2f);
			}

			atmosphereRoutineEnabled = false;

		}
			
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
		
		private void logToState(string logText) {
			stateInfos.val = stateInfos.val + ( stateInfos.val != "" ? "\n" : "" ) + logText;
		}
		
		private static void disableScrollOnText(UIDynamicTextField target) {
			ScrollRect targetSR = target.UItext.transform.parent.transform.parent.transform.parent.GetComponent<ScrollRect>();
			if( targetSR != null ) {
				targetSR.horizontal = false;
				targetSR.vertical = false;
			}
		}
		
		private void createStaticDescriptionText( string DescTitle, string DescText, bool rightSide, int fieldHeight ) {
			JSONStorableString staticDescString = new JSONStorableString(DescTitle,DescText);
			UIDynamicTextField staticDescStringField = CreateTextField(staticDescString, rightSide);
			staticDescStringField.backgroundColor = new Color(1f, 1f, 1f, 0f);
			LayoutElement sdsfLayout = staticDescStringField.GetComponent<LayoutElement>();
			sdsfLayout.preferredHeight = sdsfLayout.minHeight = fieldHeight;
			staticDescStringField.height = fieldHeight;
			disableScrollOnText(staticDescStringField);
		}

		
        // Recursive function searching for a transform
        private Transform getCUARoot( Transform parent ) {
            foreach( Transform child in parent )
            {
				if( child.name.Contains("VAMANeighbors") ) {
					return child;
				} else {
					Transform childSearch = getCUARoot(child);
					if (childSearch != null) return childSearch;
				}
            }
        
            return null;
        }
		
		private Transform getObjectNamed( Transform parent, string objName ) {
            foreach( Transform child in parent )
            {
				if( child.name == objName ) {
					return child;
				} else {
					Transform childSearch = getObjectNamed(child,objName);
					if (childSearch != null) return childSearch;
				}
            }
        
            return null;
        }
		
        private Transform toggleDebugElems( Transform parent, bool enabled ) {
            foreach( Transform child in parent )
            {
				if( child.name == "debug_elem" ) {
					child.gameObject.SetActive(enabled);
				}
				toggleDebugElems(child, enabled);
            }
        
            return null;
        }
		
		// **************************
		// Time to cleanup !
		// **************************
		void OnDestroy() {
			// Cleaning assetbundles
			AssetLoader.DoneWithAssetBundleFromFile(PLUGIN_PATH + "/vamatmosphere_neighbors.audiobundle");

		}
		
		// ***********************************************************
		// EXTERNAL TOOLS - Thank you great coders for your content!
		// ***********************************************************
		
		// *********** MacGruber_Utils.cs START *********************
		// Get directory path where the plugin is located. Based on Alazi's & VAMDeluxe's method.
		public static string GetPluginPath(MVRScript self)
		{
			string id = self.name.Substring(0, self.name.IndexOf('_'));
			string filename = self.manager.GetJSON()["plugins"][id].Value;
			return filename.Substring(0, filename.LastIndexOfAny(new char[] { '/', '\\' }));
		}
				
		// Get path prefix of the package that contains our plugin.
		public static string GetPackagePath(MVRScript self)
		{
			string id = self.name.Substring(0, self.name.IndexOf('_'));
			string filename = self.manager.GetJSON()["plugins"][id].Value;
			int idx = filename.IndexOf(":/");
			if (idx >= 0)
				return filename.Substring(0, idx+2);
			else
				return string.Empty;
		}
				
		// Check if our plugin is running from inside a package
		public static bool IsInPackage(MVRScript self)
		{
			string id = self.name.Substring(0, self.name.IndexOf('_'));
			string filename = self.manager.GetJSON()["plugins"][id].Value;
			return filename.IndexOf(":/") >= 0;
		}


	}
}